<?php
namespace Ep\Woo\Plugin\Category;

use Ep\Wp\Plugin\PluginInterface;
use WP_Post;

/**
 * Automatski dodaje kategoriju proizvoda u odabrane izbornike.
 *
 * @package Ep\Woo\Category
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-woocommerce
 */
class AutoMenu implements PluginInterface
{
    /**
     * Dodaje akcije i filtere u WordPress.
     *
     * @return void
     */
    public function __invoke(): void
    {
        if (defined('WP_ADMIN')) {
            add_action('admin_enqueue_scripts', [$this, 'admin_enqueue_scripts']);
            add_action('wp_update_nav_menu', [$this, 'wp_update_nav_menu']);
        }

        if (defined('WP_ADMIN') || defined('REST_REQUEST')) {
            add_action('created_product_cat', [$this, 'created_product_cat']);
            add_action('delete_product_cat', [$this, 'delete_product_cat']);
        }
    }

    /**
     * Dodaje front-end resurse za admin.
     *
     * @param string $hook_suffix
     */
    public function admin_enqueue_scripts(string $hook_suffix): void
    {
        if ('nav-menus.php' !== $hook_suffix) {
            return;
        }

        global $nav_menu_selected_id;
        $options = (array) get_option('nav_menu_options');

        wp_enqueue_script('woo-auto-menu', 'woo-auto-menu.js', ['jquery'], 'ep-asset');
        wp_localize_script('woo-auto-menu', 'ep_woo', [
            'auto_add_product_cats' => in_array($nav_menu_selected_id, (array) $options['auto_add_product_cats'] ?? []),
        ]);
    }

    /**
     * Okida se nakon što uspješno spremimo navigacijski izbornik.
     *
     * @param int $menu_id
     */
    public function wp_update_nav_menu(int $menu_id): void
    {
        $options = (array) get_option('nav_menu_options');

        if (!isset($options['auto_add_product_cats'])) {
            $options['auto_add_product_cats'] = [];
        }

        $autoAdd = !empty($_POST['auto-add-product-cats']);

        if ($autoAdd) {
            $options['auto_add_product_cats'][] = $menu_id;
            $options['auto_add_product_cats'] = array_unique($options['auto_add_product_cats']);
        } else {
            $index = array_search($menu_id, $options['auto_add_product_cats']);

            if (isset($options['auto_add_product_cats'][$index])) {
                unset($options['auto_add_product_cats'][$index]);
            }
        }

        update_option('nav_menu_options', $options);
    }

    /**
     * Okida se nakon što je dodana nova kategorija proizvoda.
     *
     * @param int $term_id
     */
    public function created_product_cat(int $term_id): void
    {
        $options = (array) get_option('nav_menu_options');

        if (
            !isset($options['auto_add_product_cats']) ||
            !is_array($options['auto_add_product_cats']) ||
            empty($options['auto_add_product_cats'])
        ) {
            return;
        }

        $category = get_term($term_id);

        foreach ($options['auto_add_product_cats'] as $menuId) {
            $navItemArgs = [
                'menu-item-object' => 'product_cat',
                'menu-item-object-id' => $category->term_id,
                'menu-item-type' => 'taxonomy',
                'menu-item-status' => 'publish',
            ];

            if ($category->parent > 0) {
                /** @var WP_Post[]|object[] $menuNavItems */
                $menuNavItems = wp_get_nav_menu_items($menuId) ?: [];

                foreach ($menuNavItems as $navItem) {
                    if ('product_cat' === $navItem->object && $category->parent == $navItem->object_id) {
                        $navItemArgs['menu-item-parent-id'] = $navItem->ID;
                        break;
                    }
                }
            }

            wp_update_nav_menu_item($menuId, 0, $navItemArgs);
        }
    }

    /**
     * Okida se nakon što je obrisana kategorija proizvoda.
     *
     * @param int $term
     */
    public function delete_product_cat(int $term): void
    {
        $options = (array) get_option('nav_menu_options');

        if (!isset($options['auto_add_product_cats']) || !is_array($options['auto_add_product_cats'])) {
            return;
        }

        foreach ($options['auto_add_product_cats'] as $menuId) {
            /** @var WP_Post[]|object[] $menuNavItems */
            $menuNavItems = wp_get_nav_menu_items($menuId) ?: [];

            foreach ($menuNavItems as $navItem) {
                if ('product_cat' === $navItem->object && $term == $navItem->object_id) {
                    wp_delete_post($navItem->ID);
                }
            }
        }
    }
}